/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.cdm.api.application;

import java.util.HashSet;
import java.util.Set;

import eu.etaxonomy.cdm.model.common.CdmBase;

/**
 * @author cmathew
 * @date 7 Apr 2015
 */
public class CdmChangeEvent {

    public enum Action {
        Create,
        Update,
        Delete
    }

    private final Action action;
    private final Set<CdmBase> changedObjects;
    private final Class<?> sourceType;
    private Class<?> entityType;
    private Object source;

    public CdmChangeEvent(Action action, Set<CdmBase> changedObjects, Class<?> sourceType) {
        this.action = action;
        this.changedObjects = changedObjects;
        this.sourceType = sourceType;
    }

    public CdmChangeEvent(Action action, CdmBase changedObject, Class<?> sourceType) {
        this.action = action;
        changedObjects = new HashSet<CdmBase>();
        changedObjects.add(changedObject);
        this.sourceType = sourceType;
    }

    public CdmChangeEvent(Action action, Set<CdmBase> changedObjects, Class sourceType, Class entityType) {
        this(action, changedObjects, sourceType);
        this.entityType = entityType;
    }

    public CdmChangeEvent(Action action, Set<CdmBase> changedObjects, Object source) {
        this(action, changedObjects, source.getClass());
        this.source = source;
    }

    public Action getAction() {
        return action;
    }

    public Set<? extends CdmBase> getChangedObjects() {
        return changedObjects;
    }

    public Class<?> getSourceType() {
        return sourceType;
    }

    public Object getSource() {
        return source;
    }

    public Class<?> getEntityType() {
        return entityType;
    }

    public boolean hasChanges() {
        return changedObjects != null && !changedObjects.isEmpty();
    }
}
