/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.cdm.api.application;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author cmathew
 * @date 18 Jun 2015
 */
public class CdmDataChangeService implements ICdmDataChangeService {

    protected final Set<ICdmChangeListener> listeners;

    private final List<CdmChangeEvent> currentEvents;

    public CdmDataChangeService() {
        listeners = new HashSet<ICdmChangeListener>();
        currentEvents = new ArrayList<CdmChangeEvent>();
    }

    @Override
    public void register(ICdmChangeListener listener) {
        listeners.add(listener);
    }

    @Override
    public void unregister(ICdmChangeListener listener) {
        listeners.remove(listener);
    }

    @Override
    public void addEvent(CdmChangeEvent event) {
        currentEvents.add(event);
    }

    @Override
    public void fireCurrentChangeEvents(boolean async) {
        try {
            for(CdmChangeEvent event : currentEvents) {
                fireChangeEvent(event,async);
            }
        } finally {
            currentEvents.clear();
        }
    }

    @Override
    public void fireChangeEvent(final CdmChangeEvent event, boolean async) {

        for(final ICdmChangeListener listener : listeners) {
            listener.onChange(event);
        }
    }

    @Override
    public void clear() {
        listeners.clear();
        currentEvents.clear();
    }
}
