/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.remoting.cache;

import java.util.HashSet;
import java.util.Set;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.ICommonService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.api.service.dto.CdmEntityIdentifier;
import eu.etaxonomy.cdm.cache.CdmEntityCacheKey;
import eu.etaxonomy.cdm.cache.CdmTransientEntityCacher;
import eu.etaxonomy.cdm.model.common.CdmBase;

/**
 * Note: The methods of this class were originally found in the {@link CdmTransientEntityCacher}.
 * Once that class was moved to cdmlib it was not possible to move these methods, too,
 * because of the current dependency on CdmApplicationState.
 * In future it should be tried to move them also to cdmlib by replacing the
 * {@link CdmApplicationState} dependency e.g. by adding {@link ICommonService} as parameter
 * to the method  {@link #load(CdmEntityIdentifier)}
 *
 * @author k.luther
 * @date 15.03.2018
 */
public class ConversationalTransientEntityCacher extends CdmTransientEntityCacher {

    public ConversationalTransientEntityCacher(Object sessionOwner) {
        super(sessionOwner);
    }
    private CdmBase load(CdmEntityIdentifier cei) {
        return CdmApplicationState.getCommonService().findWithUpdate(cei.getCdmClass(), cei.getId());
    }

    public UpdateResult load(UpdateResult result, boolean update) {
        // probably a good time to broadcast to other sessions

        Set<CdmBase> updatedObjects = result.getUpdatedObjects();
        if (result.getCdmEntity()!= null){
            updatedObjects.add(result.getCdmEntity());
        }
        Set<CdmBase> reloadedObjects = new HashSet<>();
        Set<CdmEntityIdentifier> updatedCdmIds = result.getUpdatedCdmIds();
        boolean updatedCdmIdsIsEmpty = updatedCdmIds.isEmpty();

        // if the cdm identifier set contains identifiers of objects already
        // present in the updated objects set remove them
        for(CdmBase updatedObject : updatedObjects) {
            if(updatedObject != null) {
                if (exists(new CdmEntityCacheKey<>(updatedObject))){
                    CdmEntityIdentifier cdmEntityIdentifier = CdmEntityIdentifier.NewInstance(updatedObject);
                    if(!updatedCdmIdsIsEmpty && updatedCdmIds.contains(cdmEntityIdentifier)) {
                        updatedCdmIds.remove(cdmEntityIdentifier);
                    }
                }
                CdmBase cachedObj = load(updatedObject, update);
                reloadedObjects.add(cachedObj);
            }
        }

        // remote load cdm identifiers of objects which already exist
        // in the cache
        for(CdmEntityIdentifier id : updatedCdmIds) {
            @SuppressWarnings({ "rawtypes", "unchecked" })
            CdmEntityCacheKey<?> cacheKey = new CdmEntityCacheKey(id.getCdmClass(), id.getId());
            if(exists(cacheKey)) {
                reloadedObjects.add(load(id));
            }
        }
        updatedObjects.clear();
        result.addUpdatedObjects(reloadedObjects);
        return result;
    }
}
