/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.remoting.server;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.text.StringEscapeUtils;

import eu.etaxonomy.cdm.config.CdmPersistentXMLSource.CdmSourceProperties;
import eu.etaxonomy.cdm.database.CdmPersistentDataSource;
import eu.etaxonomy.cdm.persistence.utils.CdmPersistenceUtils;

/**
 * @author cmathew
 * @date 11 Nov 2015
 */
public class CdmServerUtils {

    public static String convertEditorToServerConfig() {
        StringBuilder configStringBuilder = new StringBuilder();
        configStringBuilder.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?> " + System.lineSeparator());
        configStringBuilder.append("<!-- DO NOT EDIT THIS FILE MANUALLY. -->");
        configStringBuilder.append("<!-- It is created by the Taxonomic Editor for launching a managed CDM Server. -->");
        configStringBuilder.append("<beans xmlns=\"http://www.springframework.org/schema/beans\"" + System.lineSeparator());
        configStringBuilder.append("  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"" + System.lineSeparator());
        configStringBuilder.append("  xmlns:tx=\"http://www.springframework.org/schema/tx\"" + System.lineSeparator());
        configStringBuilder.append("  xmlns:context=\"http://www.springframework.org/schema/context\"" + System.lineSeparator());
        configStringBuilder.append("  xsi:schemaLocation=\"http://www.springframework.org/schema/beans   http://www.springframework.org/schema/beans/spring-beans-4.3.xsd" + System.lineSeparator());
        configStringBuilder.append("  http://www.springframework.org/schema/context   http://www.springframework.org/schema/context/spring-context-4.3.xsd" + System.lineSeparator());
        configStringBuilder.append("  http://www.springframework.org/schema/tx   http://www.springframework.org/schema/tx/spring-tx-4.3.xsd" + System.lineSeparator());
        configStringBuilder.append("\">" + System.lineSeparator());
        configStringBuilder.append(" <bean id=\"dataSourceProperties\" class=\"eu.etaxonomy.cdm.remote.config.DataSourceProperties\">" + System.lineSeparator());
        configStringBuilder.append("   <property name=\"propsMap\">" + System.lineSeparator());
        configStringBuilder.append("       <map/>" + System.lineSeparator());
        configStringBuilder.append("   </property>" + System.lineSeparator());
        configStringBuilder.append(" </bean>" + System.lineSeparator());

        for(CdmPersistentDataSource dataSource : CdmPersistentDataSource.getAllDataSources()) {
            String beanId = xmlNCNamefrom(dataSource.getName());
            String user = dataSource.getUsername();
            String password = dataSource.getPassword();
            String driverClass = dataSource.getCdmSourceProperty(CdmSourceProperties.DRIVER_CLASS);
            String jdbcUrl = StringEscapeUtils.escapeXml10(dataSource.getCdmSourceProperty(CdmSourceProperties.URL));
            configStringBuilder.append(" <bean id=\""  + beanId + "\" lazy-init=\"true\" class=\"com.mchange.v2.c3p0.ComboPooledDataSource\">" + System.lineSeparator());
            configStringBuilder.append("   <property name=\"driverClass\" value=\"" + driverClass + "\"/>" + System.lineSeparator());
            configStringBuilder.append("   <property name=\"user\" value=\"" + user + "\"/>" + System.lineSeparator());
            configStringBuilder.append("   <property name=\"password\" value=\"" + password + "\"/>" + System.lineSeparator());
            configStringBuilder.append("   <property name=\"jdbcUrl\" value=\"" + jdbcUrl + "\"/>" + System.lineSeparator());
            configStringBuilder.append(" </bean>" + System.lineSeparator());

        }
        configStringBuilder.append("</beans>" + System.lineSeparator());

        return configStringBuilder.toString();
    }

    /**
     * Assures that the datasource name is converted into a valid XML type NCName.
     *
     * Prefixes the string and replaces all forbidden characters by '_'.
     *
     * @param dataSource
     * @return
     */
    public static String xmlNCNamefrom(String beanName) {
        return "MGD_" + beanName.replaceAll("[^a-zA-Z0-9]", "_");
    }

    public static File writeManagedServerConfig(String config, String fileName) throws IOException {
        //TODO we need a pass-through class for CdmPersistenceUtils once cdmlib-persistence is not available anymore on classpath)
        File managedConfigFile = new File(CdmPersistenceUtils.getWritableResourceDir(), fileName);
        if(managedConfigFile.exists()) {
            managedConfigFile.delete();
        }
        if(managedConfigFile.createNewFile()) {
            FileUtils.writeStringToFile(managedConfigFile, config);
        }
        return managedConfigFile;
    }
}