/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.remoting.source;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.config.CdmSourceException;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.metadata.CdmMetaDataPropertyName;

/**
 * Class representing a CDM remote source
 */
public class CdmRemoteLocalhostSource extends CdmRemoteSourceBase {

    private String datasourceName;
    private ICdmDataSource datasource;

	/**
	 * Creates a new instance of CdmRemoteLocalhostSource
	 */
	 public static CdmRemoteLocalhostSource NewInstance(String name, String server, int port,
	          ICdmDataSource datasource, String datasourceName) {
        return new CdmRemoteLocalhostSource(name, server, port, datasource, datasourceName);
    }

// **************** CONSTRUCTURO **************************/

	 private CdmRemoteLocalhostSource(String name, String server, int port, ICdmDataSource datasource, String datasourceName) {
		super(name, server, port);
		this.datasource = datasource;
		this.datasourceName = datasourceName;
	}

// ***************** GETTER ***************/

    public String getDatasourceName() {
        return datasource.getName();
    }

    @Override
    public String getContext() {
        return getDatasourceName();
    }

    @Override
    public Map<CdmMetaDataPropertyName, String> getMetaDataMap() throws CdmSourceException {
        return datasource.getMetaDataMap();
    }

    @Override
    public String getDbSchemaVersion() throws CdmSourceException {
        return datasource.getDbSchemaVersion();
    }

    @Override
    public boolean isDbEmpty() throws CdmSourceException {
        return datasource.isDbEmpty();
    }

    @Override
    public boolean checkConnection() throws CdmSourceException {
        return datasource.checkConnection();
    }

    @Override
    public String toString() {
        String result = CdmUtils.concat("/", getName(), datasourceName);
        if (StringUtils.isBlank(result)){
            return super.toString();
        }else{
            return result;
        }
    }
}
