/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.remoting.source;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import eu.etaxonomy.cdm.api.application.CdmApplicationRemoteConfiguration;
import eu.etaxonomy.cdm.api.service.IMetadataService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.config.CdmSourceException;
import eu.etaxonomy.cdm.model.metadata.CdmMetaDataPropertyName;

/**
 * Class representing a CDM remote source
 */
public class CdmRemoteSource extends CdmRemoteSourceBase {

    protected static final String DEFAULT_CONTEXT_PATH = "";

    private String contextPath;
    private IMetadataService metadataService;

	/**
	 * Creates a new instance of CdmRemoteSource
	 */
	public static CdmRemoteSource NewInstance() {
		return new CdmRemoteSource(DEFAULT_NAME, DEFAULT_SERVER, DEFAULT_PORT, DEFAULT_CONTEXT_PATH);
	}

	/**
	 * Creates a new instance of CdmRemoteSource
	 *
	 * @param name
	 * @param server
	 * @param port
	 * @param contextPath
	 * @return
	 */
	public static CdmRemoteSource NewInstance(String name, String server, int port, String contextPath) {
		return new CdmRemoteSource(name, server, port, contextPath);
	}

	/**
	 * Creates a new CdmRemoteSource
	 *
	 * @param name
	 * @param server
	 * @param port
	 * @param contextPath
	 */
	protected CdmRemoteSource(String name, String server, int port, String contextPath) {
		super(name, server, port);
		this.contextPath = contextPath;
	    metadataService = CdmApplicationRemoteConfiguration.getMetadataService(this);
	}


    @Override
    public String getDbSchemaVersion() throws CdmSourceException {
        return metadataService.getDbSchemaVersion();
    }

    @Override
    public boolean isDbEmpty() throws CdmSourceException {
        return metadataService.isDbEmpty();
    }

    @Override
    public boolean checkConnection() throws CdmSourceException {
        // assuming that database service works implies
        // the connection is up
        // if no exception is thrown then we assume that the
        // connection is up
        // FIXME:Remoting is this really correct?
        metadataService.getDbSchemaVersion();

        return true;
    }

    @Override
    public Map<CdmMetaDataPropertyName, String> getMetaDataMap() throws CdmSourceException {
        return metadataService.getCdmMetadataMap();
    }


    @Override
    public String getContext() {
        return contextPath;
    }

    /**
     * @see #setContextPath(String)
     */
    public String getContextPath() {
        return contextPath;
    }

    /**
     * Sets the context path.
     * e.g. for 'http://127.0.0.1:8080/col/remoting/common.service', the
     * context path would be 'col'
     *
     * @param contextPath
     */
    public void setContextPath(String contextPath) {
        this.contextPath = contextPath;
    }

    @Override
    public String toString() {
        String result = CdmUtils.concat("/", getName(), contextPath);
        if (StringUtils.isBlank(result)){
            return super.toString();
        }else{
            return result.replace("cdmserver/", "");
        }
    }

}
