/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.remoting.source;

import org.apache.commons.lang3.StringUtils;

import eu.etaxonomy.cdm.config.CdmSource;

/**
 * Base class representing a CDM remote source.
 * This class handles all the configuration relating to the remoting aspect of
 * a CDM source.
 */
public abstract class CdmRemoteSourceBase extends CdmSource implements ICdmRemoteSource {

	protected static final String DEFAULT_NAME = "default";
	protected static final String DEFAULT_SERVER = "127.0.0.1";
	protected static final int DEFAULT_PORT = 8080;

	private String baseUrl;

	/**
	 * Constructs a CdmRemoteSourceBase object with default values.
	 */
	protected CdmRemoteSourceBase() {
		setName(DEFAULT_NAME);
		setServer(DEFAULT_SERVER);
		setPort(DEFAULT_PORT);
	}

	/**
	 * Constructs a CdmRemoteSourceBase
	 *
	 * @param name
	 * @param server
	 * @param port
	 * @param contextPath
	 * @param nomenclaturalCode
	 */
	public CdmRemoteSourceBase(String name, String server, int port) {
		setName(name);
		setServer(server);
		setPort(port);
	}

	/**
	 * @see #setBaseUrl(String)
	 */
	@Override
	public String getBaseUrl() {
		return baseUrl;
	}

	/**
	 * Sets the base url for the http-invoker services as listed in
	 * httpInvokerServicesClients.xml.
	 * e.g. for 'http://127.0.0.1:8080/col/remoting/common.service', the
	 * base url would be 'http://127.0.0.1:8080/col'
	 *
	 * @param baseUrl
	 */
	public void setBaseUrl(String baseUrl) {
		this.baseUrl = baseUrl;
	}

    @Override
    public String getConnectionMessage() {
        return "Connecting to Remote CDM Instance " + getName() + ":" + getPort() + "/" + getContext();
    }

    @Override
    public String toString() {
        String result = getName();
        if (StringUtils.isBlank(result)){
            return super.toString();
        }else{
            return result.replace("cdmserver/", "");
        }
    }
}
