/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.service;

import java.io.IOException;

import org.apache.commons.codec.binary.Base64;
import org.apache.http.client.methods.HttpPost;
import org.springframework.remoting.httpinvoker.HttpInvokerClientConfiguration;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;

/**
 * @author cmathew
 * @date 27 Jan 2015
 */
public class AuthenticatingHttpInvokerRequestExecutor extends TimestampingHttpInvokerRequestExecutor {

    @Override
    protected HttpPost createHttpPost(HttpInvokerClientConfiguration config) throws IOException {
    	HttpPost postMethod = super.createHttpPost(config);

        SecurityContext securityContext = SecurityContextHolder.getContext();
        if(securityContext != null) {
            Authentication auth = securityContext.getAuthentication();
            if ((auth != null) && (auth.getName() != null) &&
                    (auth.getCredentials() != null)) {
                String base64 = auth.getName() + ":" + auth.getCredentials().toString();
                postMethod.setHeader("Authorization", "Basic " +
                        new String(Base64.encodeBase64(base64.getBytes())));
            }
        }
        return postMethod;
    }
}
