/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.service;

import java.util.UUID;

import eu.etaxonomy.cdm.model.common.CdmBase;

/**
 * @author cmathew
 * @date 14 Oct 2014
 */
public interface ICachedCommonService {

    public CdmBase find(Class<? extends CdmBase> clazz, int id);

    /**
     * Initializes a collection or map.
     *
     * @param ownerUuid uuid of owner cdm entity
     * @param fieldName field name of collection or map
     * @return initialised collection or map
     */
    public Object initializeCollection(UUID ownerUuid, String fieldName);

    /**
     * Checks if a collection or map is empty.
     *
     * @param ownerUuid uuid of owner cdm entity
     * @param fieldName field name of collection or map
     * @return true if the collection of map is empty, else false
     */
    public boolean isEmpty(UUID ownerUuid, String fieldName);

    /**
     * Returns the size of requested collection or map.
     *
     * @param ownerUuid uuid of owner cdm entity
     * @param fieldName field name of collection or map
     * @return the size of the persistent collection
     */
    public int size(UUID ownerUuid, String fieldName);

    /**
     * Checks whether an object is contained within a persistent collection.
     *
     * @param ownerUuid uuid of owner cdm entity
     * @param fieldName field name of collection or map
     * @param element the element to check for
     * @return true if the element exists in the collection, false o/w
     */
    public boolean contains(UUID ownerUuid, String fieldName, Object element);

    /**
     * Checks whether an index object exists within a persistent collection
     * (usually a map)
     *
     * @param ownerUuid uuid of owner cdm entity
     * @param fieldName field name of map
     * @param key the index object to look for.
     * @return true if the index object exists in the collection, false o/w
     */
    public boolean containsKey(UUID ownerUuid, String fieldName, Object key);

    /**
     * checks whether an value object exists within a persistent collection
     * (usually a map)
     *
     * @param ownerUuid uuid of owner cdm entity
     * @param fieldName field name of map
     * @param key the value object to look for.
     * @return true if the value object exists in the collection, false o/w
     */
    public boolean containsValue(UUID ownerUuid, String fieldName, Object element);
}
