/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.service;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.remoting.support.RemoteInvocation;
import org.springframework.remoting.support.RemoteInvocationResult;

import eu.etaxonomy.cdm.api.cache.CdmServiceCacher;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermType;


/**
 * Note by AM (2022-06-15): This caches terms. But it seems that only the method
 * "listByTermType" from term service is cached.
 */
public class RemoteInvocationTermCacher implements IRemoteInvocationTermCacher {

    private static final Logger logger = LogManager.getLogger(RemoteInvocationTermCacher.class);

	private static Map<TermType, RemoteInvocationResult> termTypeMap = new HashMap<>();

	private static CdmServiceCacher cdmServiceCacher;

	public static void setDefaultCacher(CdmServiceCacher csc) {
        cdmServiceCacher = csc;
    }

	@Override
	public void cacheTerms(RemoteInvocation remoteInvoc, RemoteInvocationResult remoteInvocResul) {
	    if(cdmServiceCacher != null) {
	        if(remoteInvoc.getMethodName().equals("listByTermType")) {
	            if(remoteInvoc.getArguments()[1] == null) {
	                Set<DefinedTermBase<?>> terms = new HashSet<>();
	                if(remoteInvocResul.getValue() != null) {
	                    terms.addAll((List<DefinedTermBase<?>>)remoteInvocResul.getValue());

	                    for(DefinedTermBase<?> term : terms) {
	                        cdmServiceCacher.load(term);
	                    }
	                    termTypeMap.put((TermType)remoteInvoc.getArguments()[0], remoteInvocResul);
	                }
	            }
	        }
	    } else {
	        logger.info("Default CdmServiceCacher is null. Cannot cache terms");
	    }
	}

	@Override
	public  RemoteInvocationResult termsFromCache(RemoteInvocation ri) {
		return termTypeMap.get(ri.getArguments()[0]);
	}
}