package eu.etaxonomy.taxeditor.session;

import java.util.Collection;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.IService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.cache.EntityCacherDebugResult;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.dto.MergeResult;
import net.sf.ehcache.statistics.FlatStatistics;

public interface ICdmEntitySession {

    public ICdmEntitySessionEnabled<?> getOwner();

    public FlatStatistics getCacheStatistics();

	public  <O extends Object> O load(O obj, boolean update);

	/**
	 * @see eu.etaxonomy.cdm.cache.CdmTransientEntityCacher#load(Collection, boolean)
	 */
	public  <T extends CdmBase> T load(T cdmBase, boolean update);

	public  UpdateResult load(UpdateResult updateResult, boolean update);

	public  <T extends CdmBase> MergeResult<T> load(MergeResult<T> mergeResult, boolean update);

	public <T extends CdmBase> EntityCacherDebugResult debug(T cdmBase);

	public <T extends CdmBase> EntityCacherDebugResult debug(Collection<T> cdmBase, boolean includeIgnored);

	public <T extends CdmBase> EntityCacherDebugResult debug(boolean includeIgnored);

	public <T extends CdmBase> Collection<T> load(Collection<T> cdmBaseList, boolean update);

	public  void setEntitiesAsLatest();

	public <T extends CdmBase> Collection<T> getRootEntities();

	public  void dispose();

    public void bind();

//    public void addEvent(ICdmBase cdmBase, Set<CdmBase> affectedObjects, EventType eventType);

    public  <T extends CdmBase> T remoteLoad(IService<T> service, UUID uuid);

    public  <T extends CdmBase> T remoteLoad(IService<T> service, UUID uuid, List<String> propertyPaths);

    public <T extends CdmBase> UUID remoteSave(IService<T> service, T cdmBase);

    public <T extends CdmBase> T remoteUpdate(IService<T> service, T cdmBase);


//    /**
//     * @param cdmBase
//     * @param affectedObjects
//     */
//    public <T extends ICdmBase> void update(T cdmBase, Set<CdmBase> affectedObjects);
//
//    /**
//     * @param cdmBase
//     * @param affectedObjects
//     */
//    public <T extends ICdmBase> void delete(T cdmBase, Set<CdmBase> affectedObjects);
//
//    /**
//     * @param cdmBase
//     * @param affectedObjects
//     */
//    public <T extends ICdmBase> void delete(Set<T> cdmBases, Set<CdmBase> affectedObjects);


    public boolean isActive();

//    /**
//     * @param cdmBase
//     * @param affectedObject
//     */
//    public <T extends ICdmBase> void update(T cdmBase, CdmBase affectedObject);

    public <T extends CdmBase> void update();

    public List<String> getPropertyPaths(Object obj);

    public void addNewCdmEntity(CdmBase newEntity);

}