/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.session;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.IService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.cache.EntityCacherDebugResult;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.dto.MergeResult;
import net.sf.ehcache.statistics.FlatStatistics;

/**
 * @author cmathew
 * @date 30 Jul 2015
 */
public class NullSession implements ICdmEntitySession {

    private ICdmEntitySessionEnabled sessionOwner;
    private final CdmEntitySessionManager cdmEntitySessionManager;

    public NullSession(ICdmEntitySessionEnabled sessionOwner, CdmEntitySessionManager cdmEntitySessionManager) {
        this.sessionOwner = sessionOwner;
        this.cdmEntitySessionManager = cdmEntitySessionManager;
    }

    public void setOwner(ICdmEntitySessionEnabled sessionOwner) {
        this.sessionOwner = sessionOwner;
    }

    @Override
    public ICdmEntitySessionEnabled getOwner() {
        return sessionOwner;
    }

    @Override
    public FlatStatistics getCacheStatistics() {
        return null;
    }

    @Override
    public <O> O load(O obj, boolean update) {
        return obj;
    }

    @Override
    public <T extends CdmBase> T load(T cdmBase, boolean update) {
        return cdmBase;
    }

    @Override
    public UpdateResult load(UpdateResult updateResult, boolean update) {
        return updateResult;
    }

    @Override
    public <T extends CdmBase> EntityCacherDebugResult debug(T cdmBase) {
        return null;
    }

    @Override
    public <T extends CdmBase> EntityCacherDebugResult debug(Collection<T> cdmBase, boolean includeIgnored) {
        return null;
    }

    @Override
    public <T extends CdmBase> EntityCacherDebugResult debug(boolean includeIgnored) {
        return null;
    }

    @Override
    public <T extends CdmBase> Collection<T> load(Collection<T> cdmBaseList, boolean update) {
        return cdmBaseList;
    }

    @Override
    public void setEntitiesAsLatest() {

    }

    @Override
    public <T extends CdmBase> Collection<T> getRootEntities() {
        return null;
    }

    @Override
    public void dispose() {
        cdmEntitySessionManager.remove(sessionOwner);
    }

    @Override
    public void bind() {
        cdmEntitySessionManager.bindNullSession();
    }

    @Override
    public <T extends CdmBase> T remoteLoad(IService<T> service, UUID uuid) {
        T cdmBase = service.load(uuid);
        return load(cdmBase, false);
    }

    @Override
    public <T extends CdmBase> T remoteLoad(IService<T> service, UUID uuid, List<String> propertyPaths) {
        T cdmBase = service.load(uuid, propertyPaths);
        return load(cdmBase, false);
    }

    @Override
    public <T extends CdmBase> UUID remoteSave(IService<T> service, T cdmBase) {
        UUID uuid = service.save(cdmBase).getUuid();
        load(cdmBase,false);
        return uuid;
    }

    @Override
    public <T extends CdmBase> T remoteUpdate(IService<T> service, T cdmBase) {
        T mergedCdmBase = service.merge(cdmBase, true).getMergedEntity();
        return mergedCdmBase;
    }

    @Override
    public boolean isActive() {
        return cdmEntitySessionManager.getActiveSession() == this;
    }

    @Override
    public <T extends CdmBase> void update() {

    }

    @Override
    public List<String> getPropertyPaths(Object obj) {
        if(sessionOwner == null) {
            return null;
        }
        Map<Object, List<String>> propertyPathsMap = sessionOwner.getPropertyPathsMap();
        if(propertyPathsMap == null || propertyPathsMap.isEmpty()) {
            return null;
        }
        return propertyPathsMap.get(obj);
    }

    @Override
    public void addNewCdmEntity(CdmBase newEntity) {

    }

    @Override
    public <T extends CdmBase> MergeResult<T> load(MergeResult<T> mergeResult, boolean update) {
        return mergeResult;
    }


}
